/**
 * ajax请求函数模块
 */
import axios from 'axios'
import {showFullScreenLoading, tryHideFullScreenLoading} from './loading' // loading
import {showMessage, showNotify} from './message'
import qs from 'qs';
import {saveToken, getToken} from "./utils";

// export const BASE_URL = process.env.NODE_ENV === 'production' ? "" : '/api'
// export const UPLOAD_URL = BASE_URL + '/v1/frameworks/photos/upload';

const codeMessage = {
	661: '您已被强制下线',
	662: '您的账号已在其他地方登陆',
	666: '登录超时，请重新登录',
	200: '服务器成功返回请求的数据。',
	201: '新建或修改数据成功。',
	202: '一个请求已经进入后台排队（异步任务）。',
	204: '删除数据成功。',
	400: '服务端开小差了，请稍后',
	401: '用户没有权限访问',
	403: '请重新登录',
	404: '请求路径错误。',
	406: '请求的格式不可得。',
	410: '请求的资源被永久删除，且不会再得到的。',
	422: '当创建一个对象时，发生一个验证错误。',
	500: '服务端开小差了，请稍后',
	502: '服务端开小差了，请稍后',
	503: '服务端开小差了，请稍后',
	504: '服务端开小差了，请稍后'
};

export class AjaxRequest {
	constructor(config) {
		if (config) {
			this.onError = config.callback;
			this.timeout = config.timeout || 30 * 1000
			this.baseUrl = config.baseUrl || (process.env.NODE_ENV === 'production' ? "" : '/api');
			this.uploadUrl = this.baseUrl + config.uploadUrl;
			this.uploadFileUrl = this.baseUrl + config.uploadFileUrl;
			this.supportInternationalLanguage = config.supportInternationalLanguage;
			this.showNotify = config.showNotify || false
			this.deleteParamsFormBody = config.deleteParamsFormBody || false
		}
		this.init();
		return this;
	}

	init() {
		this.axios = axios.create({
			baseURL: this.baseUrl,
			timeout: this.timeout || 30000  // 请求超时时间
		});

		// 请求拦截器
		this.axios.interceptors.request.use(config => {
			if (config.showLoading) {
				showFullScreenLoading()
			}
			config.headers['Content-Type'] = 'application/json;charset=utf-8'
			if (config.contentType) {
				config.headers['Content-Type'] = config.contentType
			}
			const token = getToken();
			if (token) {
				config.headers.common['token'] = token
			}
			if (config.securityLevel !== undefined) {
				config.headers.common['Security-Level'] = config.securityLevel
			}
			if (config.authority !== undefined) {
				config.headers.common['authority'] = config.authority
			}
			return config;
		}, error => { // 请求错误处理
			return Promise.reject(error)
		});

		// 回复拦截器
		this.axios.interceptors.response.use(
			response => { // 成功请求到数据
				if (response.config.showLoading) {
					tryHideFullScreenLoading()
				}
				// 这里根据后端提供的数据进行对应的处理
				if (response.config.url.indexOf('/security') > -1) {
					// 保存token
					let token = response.headers.token;
					saveToken(token);
				}

				if (response.status === 200) {
					if (response.headers['content-type'].indexOf('application/json') > -1 && response.data.success) {
						return response.data
					} else if (response.headers['content-type'].indexOf('application/x-zip-compressed') > -1 ||
						response.headers['content-type'].indexOf('application/octet-stream') > -1
					) {
						this.downloadFile(response);
						return response
					} else {
						const errorMessage = response.data.message || '网络异常'
						this.showNotify ? showNotify(errorMessage, 'error') : showMessage(errorMessage, 'error')
						return Promise.reject(response)
					}
				} else {
					this.showNotify ? showNotify('网络异常', 'error') : showMessage('网络异常', 'error')
					return Promise.reject(response);
				}
			},
			error => { // 响应错误处理
				tryHideFullScreenLoading()
				if (error.response) {
					const {data, status} = error.response
					const errorMsg = data && data.message ? data.message : codeMessage[status];
					this.showNotify ? showNotify(errorMsg, 'error') : showMessage(errorMsg, 'error')
					this.onError && this.onError(status)
				}
				return Promise.reject(error)
			}
		);

	}

	request(url, method = 'get', params = {}, config = {}) {
		let ajaxConfig = {
			method: method,
			url,
			...config
		}
		const language = localStorage.getItem('language');

		if (method === 'get' || method === 'GET' || ((method === 'delete' || method === 'DELETE') && !this.deleteParamsFormBody)) {
			ajaxConfig = {
				...ajaxConfig,
				params: this.supportInternationalLanguage && language ? {
					...params,
					lang: language
				} : params,
				paramsSerializer: function (params) {
					return qs.stringify(params, {arrayFormat: 'repeat'})
				}
			}
		} else {
			ajaxConfig = {
				...ajaxConfig,
				data: params,
				params: this.supportInternationalLanguage && language ? {
					lang: language
				} : undefined,
			}
		}
		return new Promise((resolve, reject) => {
			this.axios(ajaxConfig).then(res => {
				resolve(res.data)
			}).catch(err => {
				reject(err)
			})
		})
	}

	/**
	 * 下载文件
	 * @param response
	 */
	downloadFile(response) {
		const contentDispositionBuffer = response.headers['content-disposition'].split("=");
		let fileName = 'download';
		if (contentDispositionBuffer.length === 2) {
			fileName = decodeURIComponent(contentDispositionBuffer[1]);
		}
		let downloadElement = document.createElement("a");
		let href = window.URL.createObjectURL(new Blob([response.data])); //创建下载的链接
		downloadElement.href = href;
		downloadElement.download = fileName; //下载后文件名
		downloadElement.style.display = 'none';
		document.body.appendChild(downloadElement);
		downloadElement.click(); //点击下载
		document.body.removeChild(downloadElement); //下载完成移除元素
		window.URL.revokeObjectURL(href); //释放掉blob对象
	}
}




