/**
 * Description: 登陆
 */
import {
	reqLogin,
	reqToken,
	reqSmsCode,
	reqLogout,
	reqRegister,
	reqUserInfo,
	updatePasswordBySmsCode
} from "../api/uas";
import { getPublicKey, savePublicKey, saveUserInfo } from "./utils";
import _ from "lodash";
import { baizeDecrypt, baizeEncrypt } from "./baize_coder";
import { getAuthorities } from "./uas";
import validator from "validator";

/**
 * 获取会话
 */
export const getBjToken = () => {
	return new Promise(async (resolve, reject) => {
		// 获取token
		const publicKey = await reqToken();
		if (!publicKey) {
			reject(new Error('获取会话失败，请稍后重试'));
		}
		// 本地保存公钥
		savePublicKey(publicKey);
		resolve(publicKey);
	});
}


/**
 * 获取手机验证码
 * @param phone 手机号码
 * @returns {Promise<any>}
 */
export const getBjSmsCode = (phone) => {
	return new Promise(async (resolve, reject) => {
		if (_.isEmpty(phone) || !validator.isMobilePhone(phone, ['zh-CN'])) {
			reject(new Error('请输入正确的手机号码'));
			return;
		}
		try {
			const publicKey = await getBjToken();
			// 发送ajax请求(向指定手机号发送验证码短信)
			const params = {
				phone
			}
			// 手机信息全文加密
			let encrypt = baizeEncrypt(params, publicKey);
			// 发送验证码请求
			await reqSmsCode(encrypt);
			resolve();
		} catch (error) {
			reject(error);
		}
	});
}

/**
 * 短信登录
 * @param phone
 * @param code
 * @param systemType
 */
export const bjLoginBySms = (phone, code) => {
	return new Promise(async (resolve, reject) => {
		if (_.isEmpty(phone) || !validator.isMobilePhone(phone, ['zh-CN'])) {
			reject(new Error('请输入正确的手机号码'));
			return;
		}
		if (_.isEmpty(code)) {
			reject(new Error('请输入验证码'));
			return;
		}
		// 获取publicKey
		const publicKey = getPublicKey();
		if (_.isEmpty(publicKey)) {
			reject(new Error('请重新获取验证码'));
			return;
		}
		try {
			// 登录信息全文加密
			const encryptContent = {
				identityType: 'phone',
				identifier: phone,
				code: code
			}
			let encrypt = baizeEncrypt(encryptContent, publicKey);
			// 登录
			const loginData = await reqLogin(encrypt);
			delete loginData.success
			delete loginData.message
			// 保存用户信息
			saveUserInfo(loginData);
			// 获取用户权限列表和版本
			await getAuthorities();
			resolve({
				message: '登录成功',
				data: loginData
			});
		} catch (error) {
			reject(error)
		}
	});
}


/**
 * 账号密码登录
 * @param account
 * @param password
 */
export const bjLoginByPassword = (account, password) => {
	return new Promise(async (resolve, reject) => {
		if (_.isEmpty(account) || _.isEmpty(password)) {
			reject(new Error('请输入用户名或密码'));
			return;
		}
		try {
			const publicKey = await getBjToken();
			// 登录信息全文加密
			const encryptContent = {
				identityType: 'loginname',
				identifier: account,
				credential: password
			}
			let encrypt = baizeEncrypt(encryptContent, publicKey);
			// 登录
			const loginData = await reqLogin(encrypt);
			delete loginData.success
			delete loginData.message
			// 保存用户信息
			saveUserInfo(loginData);
			// 获取用户权限列表和版本
			await getAuthorities();
			resolve({
				message: '登录成功',
				data: loginData
			});
		} catch (error) {
			reject(error)
		}
	});
}


/**
 * 手机注册账号
 * @param phone
 * @param password
 * @param code
 */
export const bjRegister = (phone, password, code) => {
	return new Promise(async (resolve, reject) => {
		if (_.isEmpty(phone) || !validator.isMobilePhone(phone, ['zh-CN'])) {
			reject(new Error('请输入正确的手机号码'));
			return;
		}
		if (_.isEmpty(password)) {
			reject(new Error('请输入密码'));
			return;
		}
		if (_.isEmpty(code)) {
			reject(new Error('请输入验证码'));
			return;
		}
		// 获取publicKey
		const publicKey = getPublicKey()
		if (_.isEmpty(publicKey)) {
			reject(new Error('请重新获取验证码'));
			return;
		}
		try {
			// 注册信息全文加密
			const params = {
				identityType: 'loginname',
				identifier: phone,
				credential: password,
				phone,
				code,
				enterpriseid: 'B80FB20EDD6034980F492C12D0E20744'   // 深圳愿景微棠账号
			}
			let encrypt = baizeEncrypt(params, publicKey);
			await reqRegister(encrypt)
			resolve({
				message: '注册成功'
			});
		} catch (error) {
			reject(error);
		}
	});
}

/**
 * 登出
 */
export const bjLogout = (requestLogout = true) => {
	return new Promise(async (resolve, reject) => {
		requestLogout && await reqLogout();
		localStorage.removeItem('token');
		localStorage.removeItem('publicKey');
		localStorage.removeItem('userInfo');
		localStorage.removeItem('authorities');
		localStorage.removeItem('authorityCode');
		resolve();
	});
}


/**
 * 向服务器获取用户信息
 * @param userKey
 * @returns {Promise<any>}
 */
export const getBjUserInfo = (userKey) => {
	return new Promise(async (resolve, reject) => {
		try {
			const data = await reqUserInfo({ userKey });
			const publicKey = getPublicKey();
			if (!data || !publicKey) {
				reject(new Error('获取用户信息失败'));
				return;
			}
			const userInfoStr = baizeDecrypt(data, publicKey);
			const userInfo = JSON.parse(userInfoStr)
			resolve(userInfo);
		} catch (error) {
			reject(new Error('获取用户信息失败'));
		}
	});
}



/**
 * 手机短信修改密码
 * @param newPassword 新密码
 * @param code 验证码
 * @returns {Promise<any>}
 */
export const bjUpdatePasswordBySmsCode = (phone, code, newPassword) => {
	return new Promise(async (resolve, reject) => {
		try {
			if (_.isEmpty(code)) {
				reject(new Error('请输入验证码'));
				return
			}
			if (_.isEmpty(newPassword)) {
				reject(new Error('请输入新密码'));
				return
			}
			// 获取publicKey
			const publicKey = getPublicKey()
			if (_.isEmpty(publicKey)) {
				reject(new Error('请重新获取验证码'));
				return;
			}
			const encryptContent = {
				phone,
				code,
				newPwd: newPassword,
				source: 'enterprise',
				enterpriseid: 'B80FB20EDD6034980F492C12D0E20744'   // 深圳愿景微棠账号
			}
			let encrypt = baizeEncrypt(encryptContent, publicKey);
			await updatePasswordBySmsCode(encrypt);
			resolve({
				message: '修改成功'
			});
		} catch (error) {
			reject(error);
		}
	});
}
